@@ -13,7 +13,7 @@ Crypt::Perl::PKCS10 - Certificate Signing Request (CSR) creation
 
     my $pkcs10 = Crypt::Perl::PKCS10->new(
 
-        key => $private_key_obj,
+        key => $key_obj,
 
         subject => [
             commonName => 'foo.com',
@@ -23,8 +23,8 @@ Crypt::Perl::PKCS10 - Certificate Signing Request (CSR) creation
         attributes => [
             [ 'extensionRequest',
                 [ 'subjectAltName',
-                    [ dNSName => 'foo.com' ],
-                    [ dNSName => 'bar.com' ],
+                    dNSName => 'foo.com',
+                    dNSName => 'bar.com',
                 ],
             ],
         ],
@@ -45,15 +45,8 @@ L<Let’s Encrypt|http://letsencrypt.org>.
 It’s also a good deal easier to use!
 
 I believe this is the only L<CPAN|http://search.cpan.org> module that
-can create CSRs for RSA, ECDSA, and Ed25519 keys. Other encryption schemes
-would not be difficult to integrate—but do any CAs accept them?
-
-=head1 ECDSA KEY FORMAT
-
-After a brief flirtation (cf. v0.13) with producing ECDSA-signed CSRs using
-explicit curve parameters, this module produces CSRs using B<named> curves.
-Certificate authorities seem to prefer this format—which makes sense since
-they only allow certain curves in the first place.
+can create CSRs for either RSA or ECDSA keys. Other encryption schemes would
+not be difficult to integrate—but do any CAs accept them?
 
 =head1 SIGNATURE DIGEST ALGORITHMS
 
@@ -64,8 +57,6 @@ ECDSA it’s the strongest SHA digest algorithm that the key allows
 
 If you need additional flexibility, let me know.
 
-(Note that Ed25519 signs an entire document rather than a digest.)
-
 =head1 CLASS METHODS
 
 =head2 new( NAME => VALUE, ... );
@@ -74,16 +65,16 @@ Create an instance of this class. Parameters are:
 
 =over 4
 
-=item * C<key> - An instance of C<Crypt::Perl::RSA::PrivateKey>,
-C<Crypt::Perl::ECDSA::PrivateKey>, or C<Crypt::Perl::Ed25519::PrivateKey>.
+=item C<key>: An instance of either
+C<Crypt::Perl::RSA::PrivateKey> or C<Crypt::Perl::ECDSA::PrivateKey>.
 If you’ve got a DER- or PEM-encoded key string, use L<Crypt::Perl::PK>
 (included in this distribution) to create an appropriate object.
 
-=item * C<subject> - An array reference of arguments into
+=item C<subject>: An array reference of arguments into
 L<Crypt::Perl::X509::Name>’s constructor.
 
-=item * C<attributes> - An array reference of arguments into
-L<Crypt::Perl::PKCS10::Attributes>’s constructor.
+=item C<attributes>: An array reference of arguments into
+L<Crypt::Perl::PKCS10::Attrbutes>’s constructor.
 
 =back
 
@@ -106,12 +97,14 @@ Currently this only seems to support RSA.
 
 =cut
 
+use Crypt::Format ();
+use Digest::SHA ();
+
 use Crypt::Perl::ASN1 ();
-use Crypt::Perl::ASN1::Signatures ();
 use Crypt::Perl::PKCS10::Attributes ();
+use Crypt::Perl::PKCS10::ASN1 ();
 use Crypt::Perl::PKCS10::Attributes ();
 use Crypt::Perl::X509::Name ();
-use Crypt::Perl::X ();
 
 use parent qw( Crypt::Perl::ASN1::Encodee );
 
@@ -120,7 +113,6 @@ use parent qw( Crypt::Perl::ASN1::Encodee );
 sub to_pem {
     my ($self) = @_;
 
-    require Crypt::Format;
     return Crypt::Format::der2pem( $self->to_der(), 'CERTIFICATE REQUEST' );
 }
 
@@ -149,7 +141,7 @@ use constant asn1_macro => 'CertificationRequest';
 sub new {
     my ($class, %opts) = @_;
 
-    my ($key, $attrs, $subject) = @opts{'key', 'attributes', 'subject'};
+    my ($key, $attrs, $subject, $sigalg) = @opts{'key', 'attributes', 'subject', 'signature_algorithm'};
 
     $subject = Crypt::Perl::X509::Name->new( @$subject );
     $attrs = Crypt::Perl::PKCS10::Attributes->new( @$attrs );
@@ -158,6 +150,7 @@ sub new {
         _key => $key,
         _subject => $subject,
         _attributes => $attrs,
+        _sigalg => $sigalg,
     };
 
     return bless $self, $class;
@@ -172,12 +165,11 @@ sub _encode_params {
     my ($sig_alg, $sig_param, $sig_func);
 
     if ($key->isa('Crypt::Perl::ECDSA::PrivateKey')) {
-        require Digest::SHA;
 
         my $bits = $key->max_sign_bits();
 
         if ($bits < 224) {
-            die Crypt::Perl::X::create('Generic', "This key is too weak ($bits bits) to make a secure PKCS #10 CSR.");
+            die "This key is too weak ($bits bits) to make a secure PKCS #10 CSR.";
         }
         elsif ($bits < 256) {
             $bits = 224;
@@ -194,36 +186,27 @@ sub _encode_params {
 
         $sig_alg = "ecdsa-with-SHA$bits";
 
-        my $fn = "sign_sha$bits";
-
         $sig_func = sub {
             my ($key, $msg) = @_;
 
-            return $key->$fn($msg);
+            return $key->sign( Digest::SHA->can("sha$bits")->($msg) );
         };
 
         $pk_der = $key->get_public_key()->to_der_with_curve_name();
     }
     elsif ($key->isa('Crypt::Perl::RSA::PrivateKey')) {
-        require Digest::SHA;
-
         $sig_alg = 'sha512WithRSAEncryption';
         $sig_param = q<>;
         $sig_func = $key->can('sign_RS512');
 
         $pk_der = $key->to_subject_public_der();
     }
-    elsif ($key->isa('Crypt::Perl::Ed25519::PrivateKey')) {
-        $sig_alg = 'ed25519';
-        $sig_func = $key->can('sign');
-        $pk_der = $key->get_public_key()->to_der();
-    }
     else {
-        die Crypt::Perl::X::create('Generic', "Key ($key) is not a recognized private key class instance!");
+        die "Key ($key) is not a recognized private key class instance!";
     }
 
-    $sig_alg = $Crypt::Perl::ASN1::Signatures::OID{$sig_alg} || do {
-        die Crypt::Perl::X::create('Generic', "Unrecognized signature algorithm OID: “$sig_alg”");
+    $sig_alg = $Crypt::Perl::PKCS10::ASN1::OID{$sig_alg} || do {
+        die "Unrecognized signature algorithm OID: “$sig_alg”";
     };
 
     my $asn1_reqinfo = Crypt::Perl::ASN1->new()->prepare( $self->ASN1() );
@@ -238,9 +221,6 @@ sub _encode_params {
     #This is a detail germane to the PKCS10 structure, not to the
     #Attributes itself (right??), so it makes sense to do the change here
     #rather than to put “[0] SET” into the ASN1 template for Attributes.
-    #
-    #“use bytes” is not necessary because we know the first character is
-    #0x31, which came from Convert::ASN1.
     substr($attr_enc, 0, 1) = chr 0xa0;
 
     my %reqinfo = (
