@@ -3,6 +3,8 @@ package OpenSSL_Control;
 use strict;
 use warnings;
 
+use Test::More;
+
 use Call::Context ();
 use File::Temp ();
 use File::Which ();
@@ -40,7 +42,7 @@ sub verify_private {
     my $ver = qx<$openssl_bin dgst -$digest_alg -prverify $dir/key -signature $dir/sig $dir/msg>;
     my $ok = $ver =~ m<OK>;
 
-    diag $ver if !$ok;
+    warn $ver if !$ok;
 
     return $ok;
 }
